@@ -23,6 +23,9 @@ class GP(Op):
     def __add__(self, other):
         return Stack([self]) + other
 
+    def __mul__(self, other):
+        return Product([self]) * other
+
     def call(self, p, e):
         """
         Returns tuple `(mean, covariance)` for locations.
@@ -116,75 +119,7 @@ class SquaredExponential(GP):
 
     def call(self, p, e):
         v = get_parameter_values(self.fa, p)
-        return None, v['sill'] * tf.exp(-e['d2'] / tf.square(v['range']))
-
-    def reg(self, p):
-        v = get_parameter_values(self.fa, p)
-        return v['range']
-
-class SquaredExponentialIntRough(GP):
-    def __init__(self, axis, start, sill='sill', range='range', scale=None, metric=None):
-        fa = dict(sill=sill, range=range)
-        self.axis = axis
-        self.start = start
-        autoinputs = scale_to_metric(scale, metric)
-        super().__init__(fa, dict(d2=autoinputs, locs1='locs1', locs2='locs2'))
-
-    def vars(self):
-        return ppp(self.fa['sill']) + ppp(self.fa['range'])
-
-    def call(self, p, e):
-        v = get_parameter_values(self.fa, p)
-        x1 = e['locs1'][..., self.axis]
-        x2 = e['locs2'][..., self.axis]
-        envelope = (tf.minimum(ed(x1, 1), ed(x2, 0)) - self.start) * v['sill']
-        c = envelope * tf.exp(-e['d2'] / tf.square(v['range']))
-        return None, c
-
-    def reg(self, p):
-        v = get_parameter_values(self.fa, p)
-        return v['range']
-
-class SquaredExponentialInt(GP):
-    def __init__(self, axis, start, sill='sill', range='range', scale=None, metric=None):
-
-        # Too much of a pain to implement a default isotropic scale,
-        # so make the user supply it.
-        assert scale is not None
-
-        self.axis = axis
-        self.start = start
-
-        # Zero out the element of scale corresponding to the axis of
-        # integration.
-        zeroed_scale = scale.copy()
-        zeroed_scale[self.axis] = 0.
-        d2 = scale_to_metric(zeroed_scale, metric)
-
-        # Include the element of scale corresponding to the axis of
-        # integration as an explicit formal argument.
-        fa = dict(sill=sill, range=range, scale_on_axis=scale[self.axis])
-
-        super().__init__(fa, dict(d2=d2, locs1='locs1', locs2='locs2'))
-
-    def vars(self):
-        return ppp(self.fa['sill']) + ppp(self.fa['range']) + ppp(self.fa['scale_on_axis'])
-
-
-    def call(self, p, e):
-        v = get_parameter_values(self.fa, p)
-        x1 = tf.pad(e['locs1'][..., self.axis] - self.start, [[1, 0]])
-        x2 = tf.pad(e['locs2'][..., self.axis] - self.start, [[1, 0]])
-
-        r = v['range'] / v['scale_on_axis']
-        sdiff = (ed(x1, 1) - ed(x2, 0)) / (r * np.sqrt(2.))
-        k = -tf.square(r) * (np.sqrt(np.pi) * sdiff * tf.math.erf(sdiff) + tf.exp(-tf.square(sdiff)))
-        k -= k[0:1, :]
-        k -= k[:, 0:1]
-        k = k[1:, 1:]
-
-        c = v['sill'] * k * tf.exp(-e['d2'] / (2. * tf.square(r)))
-        return None, c
+        return None, v['sill'] * tf.exp(-0.5 * e['d2'] / tf.square(v['range']))
 
     def reg(self, p):
         v = get_parameter_values(self.fa, p)
@@ -207,95 +142,94 @@ class GammaExponential(GP):
         v = get_parameter_values(self.fa, p)
         return v['range']
 
-class GammaExponentialInt(GP):
-    def __init__(self, axis, start, sill='sill', range='range', gamma='gamma', scale=None, metric=None):
-
-        # Too much of a pain to implement a default isotropic scale,
-        # so make the user supply it.
-        assert scale is not None
+class Wiener(GP):
+    def __init__(self, axis, start):
 
         self.axis = axis
         self.start = start
 
-        # Zero out the element of scale corresponding to the axis of
-        # integration.
-        zeroed_scale = scale.copy()
-        zeroed_scale[self.axis] = 0.
-        d2 = scale_to_metric(zeroed_scale, metric)
+        # Include the element of scale corresponding to the axis of
+        # integration as an explicit formal argument.
+        fa = dict()
+
+        super().__init__(fa, dict(locs1='locs1', locs2='locs2'))
+
+    def vars(self):
+        return []
+
+    def call(self, p, e):
+        v = get_parameter_values(self.fa, p)
+        x1 = e['locs1'][..., self.axis]
+        x2 = e['locs2'][..., self.axis]
+        k = tf.minimum(ed(x1, 1), ed(x2, 0)) - self.start
+        return None, k
+
+    def reg(self, p):
+        return 0.
+
+class IntSquaredExponential(GP):
+    def __init__(self, axis, start, range='range'):
+
+        self.axis = axis
+        self.start = start
 
         # Include the element of scale corresponding to the axis of
         # integration as an explicit formal argument.
-        fa = dict(sill=sill, range=range, gamma=gamma, scale_on_axis=scale[self.axis])
+        fa = dict(range=range)
 
-        super().__init__(fa, dict(d2=d2, locs1='locs1', locs2='locs2'))
+        super().__init__(fa, dict(locs1='locs1', locs2='locs2'))
 
     def vars(self):
-        return ppp(self.fa['sill']) + ppp(self.fa['range']) + \
-               bpp(self.fa['gamma'], 0., 2.) + ppp(self.fa['scale_on_axis'])
+        return ppp(self.fa['range'])
 
     def call(self, p, e):
         v = get_parameter_values(self.fa, p)
         x1 = tf.pad(e['locs1'][..., self.axis] - self.start, [[1, 0]])
         x2 = tf.pad(e['locs2'][..., self.axis] - self.start, [[1, 0]])
 
-        r = v['range'] / v['scale_on_axis']
+        r = v['range']
         sdiff = (ed(x1, 1) - ed(x2, 0)) / (r * np.sqrt(2.))
         k = -tf.square(r) * (np.sqrt(np.pi) * sdiff * tf.math.erf(sdiff) + tf.exp(-tf.square(sdiff)))
         k -= k[0:1, :]
         k -= k[:, 0:1]
         k = k[1:, 1:]
 
-        c = v['sill'] * k * gamma_exp(e['d2'] / tf.square(v['range']), v['gamma'])
-        return None, c
+        return None, k
 
     def reg(self, p):
-        v = get_parameter_values(self.fa, p)
-        return v['range']
+        return 0.
 
-class GammaExponentialIntExponential(GP):
-    def __init__(self, axis, start, sill='sill', range='range', gamma='gamma', scale=None, metric=None):
-
-        # Too much of a pain to implement a default isotropic scale,
-        # so make the user supply it.
-        assert scale is not None
+class IntExponential(GP):
+    def __init__(self, axis, start, range='range'):
 
         self.axis = axis
         self.start = start
 
-        # Zero out the element of scale corresponding to the axis of
-        # integration.
-        zeroed_scale = scale.copy()
-        zeroed_scale[self.axis] = 0.
-        d2 = scale_to_metric(zeroed_scale, metric)
-
         # Include the element of scale corresponding to the axis of
         # integration as an explicit formal argument.
-        fa = dict(sill=sill, range=range, gamma=gamma, scale_on_axis=scale[self.axis])
+        fa = dict(range=range)
 
-        super().__init__(fa, dict(d2=d2, locs1='locs1', locs2='locs2'))
+        super().__init__(fa, dict(locs1='locs1', locs2='locs2'))
 
     def vars(self):
-        return ppp(self.fa['sill']) + ppp(self.fa['range']) + \
-               bpp(self.fa['gamma'], 0., 2.) + ppp(self.fa['scale_on_axis'])
+        return ppp(self.fa['range'])
 
     def call(self, p, e):
         v = get_parameter_values(self.fa, p)
         x1 = tf.pad(e['locs1'][..., self.axis] - self.start, [[1, 0]])
         x2 = tf.pad(e['locs2'][..., self.axis] - self.start, [[1, 0]])
 
-        r = v['range'] / v['scale_on_axis']
-        sdiff = (ed(x1, 1) - ed(x2, 0)) / (r * np.sqrt(2.))
+        r = v['range']
+        sdiff = tf.abs(ed(x1, 1) - ed(x2, 0)) / r
         k = -tf.square(r) * (sdiff + tf.exp(-sdiff))
         k -= k[0:1, :]
         k -= k[:, 0:1]
         k = k[1:, 1:]
 
-        c = v['sill'] * k * gamma_exp(e['d2'] / tf.square(v['range']), v['gamma'])
-        return None, c
+        return None, k
 
     def reg(self, p):
-        v = get_parameter_values(self.fa, p)
-        return v['range']
+        return 0.
 
 class Noise(GP):
     def __init__(self, nugget='nugget'):
@@ -366,6 +300,32 @@ class Stack(GP):
     def reg(self, p):
         return tf.reduce_sum([part.reg(p) for part in self.parts], axis=0)
 
+class Product(GP):
+    def __init__(self, parts: List[GP]):
+        self.parts = parts
+        super().__init__({}, dict(locs1='locs1', locs2='locs2', parts=parts))
+
+    def vars(self):
+        return [p for part in self.parts for p in part.vars()]
+
+    def __mul__(self, other):
+        if isinstance(other, GP):
+            return Product(self.parts + [other])
+    
+    def call(self, p, e):
+        MM, CC = zip(*e['parts'])
+        MM = [M for M in MM if M is not None]
+        CC = [C for C in CC if C is not None]
+        M = tf.reduce_prod(MM, axis=0)
+        C = tf.reduce_prod(CC, axis=0)
+        return M, C
+
+    def report(self, p):
+        return ' '.join(part.report(p) for part in self.parts)
+
+    def reg(self, p):
+        return tf.reduce_sum([part.reg(p) for part in self.parts], axis=0)
+
 # Gamma exponential covariance function.
 @tf.custom_gradient
 def safepow(x, a):
